#ifndef PIPELINE_H
#define PIPELINE_H

#include <vector>
#include <cstddef>
#include <string>

class ShaderSource;
class Module;
class Scene;

class Pipeline
{
protected:
    /**
     * @brief every module in this vector will be used by the renderer for resizing, and rendering
     */
    std::vector<Module*> modules;
public:

    /**
     * @brief getModules is a getter used by the renderer to access the pipeline modules
     */
    std::vector<Module*>& getModules() {return modules;}

    /**
     * the destructor deletes all the modules
     */
    ~Pipeline();
};

/**
 * @brief The SimplePipeline class creates a simple graphic pipeline able to render 3D meshes
 * it is compatible with crappy rendering
 */
class SimplePipeline : public Pipeline
{
    bool isCrappy;

public:
    SimplePipeline(ShaderSource *forwardSource = NULL);

    void refreshScene(Scene *scene);
};

class DefaultPipeline : public Pipeline
{
public:
    struct Settings
    {
        bool usesDeferred;
        bool hasSkybox;
        bool hasPicking;
        bool hasBloom;
    };

    struct SourcePack
    {
        std::string forward_frag;
        std::string forward_vert;

        std::string gbuffer_frag;
        std::string gbuffer_vert;
        std::string deferred_frag;
        std::string deferred_vert;

        std::string bloom_frag;
        std::string blur_frag;
        std::string hdr_frag;
        std::string luminance_frag;
        std::string reduction_frag;
    };

    DefaultPipeline(const Settings &mySettings, const SourcePack &mySourcePack);

    void refreshScene(Scene *scene);
};

#endif // PIPELINE_H
