#include "shadersource.h"
#include <string>
#include <vector>
#include <sstream>
#include "shader.h"

ShaderSource::ShaderSource()
{
    for(int i=0; i<NB_TYPES; ++i)
        sources[i] = NULL;
}

ShaderSource::~ShaderSource()
{
    for(int i=0; i<NB_TYPES; ++i)
        if(sources[i] != NULL)
            delete(sources[i]);
}

void ShaderSource::setSource(const char *source, SourceType type)
{
    if(sources[type] != NULL)
        delete(sources[type]);
    if(source == NULL)
        sources[type] = NULL;
    else
        sources[type] = new std::string(source);
}

Shader* ShaderSource::compile(int nbDefines, const char** defines)
{
    if(sources[VERTEX] == NULL || sources[FRAGMENT] == NULL)
        return NULL;
    std::string compiledSources[NB_TYPES];
    for(int i=0; i<NB_TYPES; ++i)
    {
        if(sources[i] == NULL)
            continue;
        compiledSources[i] = preprocess(*(sources[i]), nbDefines, defines);
    }
    if(sources[GEOMETRY] != NULL)
        return new Shader(compiledSources[VERTEX], compiledSources[GEOMETRY], compiledSources[FRAGMENT]);
    else
        return new Shader(compiledSources[VERTEX], compiledSources[FRAGMENT]);
}

std::string ShaderSource::preprocess(std::string source, int nbDefines, const char** defines, const char* version)
{
    std::string header = "#version "+std::string(version);
    for(int i=0; i<nbDefines; ++i)
        header += "\n#define "+std::string(defines[i]);
    return header+"\n#line 1\n"+source;
}
