#include "glew.h"
#include <GL/gl.h>
#include <GL/glext.h>
#include <cstdio>
#include "sparrowrenderer.h"
#include "glassert.h"
#include "camera.h"
#include "framebuffer.h"
#include "module.h"
#include <chrono>
#include <string.h>
#include <iostream>
#include "pipeline.h"
#include "scene.h"
// main methods

bool SparrowRenderer::modernOpenglAvailable = false;

void SparrowRenderer::initGL(int w, int h, bool forceCrappy)
{
    glewExperimental = GL_TRUE;
    GLenum err = glewInit();
    if (GLEW_OK != err)
        fprintf(stderr, "Warning: glewInit failed!\n");
    if (!forceCrappy &&
        GLEW_ARB_vertex_program   &&
        GLEW_ARB_draw_buffers     &&
        GLEW_ARB_framebuffer_object &&
        glewIsSupported("GL_VERSION_3_3") &&
        GLEW_VERSION_3_3 &&
        strcmp("3.30",(const char *)glGetString(GL_SHADING_LANGUAGE_VERSION)) <= 0)
    {
        modernOpenglAvailable = true;
        printf("Modern OpenGL available.\n");
    }
    else
    {
        modernOpenglAvailable = false;
        fprintf(stderr, "Warning: modern OpenGL not supported!\nEnabling fallback crappy rendering mode\n");
    }

    std::cout << "OpenGL " << glGetString(GL_VERSION) << std::endl;
    std::cout << "GLSL " << glGetString(GL_SHADING_LANGUAGE_VERSION) << std::endl;
    std::cout << "Renderer " << glGetString(GL_RENDERER) << std::endl;
    std::cout << "Vendor " << glGetString(GL_VENDOR) << std::endl;

    glAssert(glEnable(GL_DEPTH_TEST));
    glAssert(glEnable(GL_CULL_FACE));
    glAssert(glEnable(GL_TEXTURE_2D));
    resizeGL(w, h);
}

void SparrowRenderer::resizeGL(int w, int h)
{
    width = w;
    height = h;
    if(scene != NULL && scene->getCamera() != NULL)
        scene->getCamera()->resize(width, height);
    if(modules != NULL)
    {
        for(Module *m : *modules)
            m->resize(w, h);
    }
}

void SparrowRenderer::renderGL()
{
    glAssert(glViewport(0, 0, width, height));
    glAssert(glClearColor(clearColor.r, clearColor.g, clearColor.b, 1));
    glAssert(glClearDepth(1.0));

    if(modules != NULL)
    {
        for(Module *m : *modules)
        {
            glFinish();
            m->renderGL(scene == NULL ? NULL : scene->getCamera(), scene);
        }
    }
    glFinish();
}

bool SparrowRenderer::isModernOpenGLAvailable()
{
    return modernOpenglAvailable;
}

// pipeline methods

void SparrowRenderer::setPipeline(Pipeline* pipeline)
{
    if(pipeline != NULL)
        modules = &(pipeline->getModules());
    else
        modules = NULL;
}

// scene methods

void SparrowRenderer::setScene(Scene* myScene)
{
    scene = myScene;
}

Scene* SparrowRenderer::getScene()
{
    return scene;
}


